#include <inttypes.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "core_map.h"

extern int print_debug;

static void DEBUG(const char *format, ...)
{
	if (print_debug) {
		va_list alist;

		va_start(alist, format);
		fputs("DEBUG: ", stderr);
		vfprintf(stderr, format, alist);
		va_end(alist);
	}
}

/* This is the list of non-empty LOAD segments in the core */
static struct core_map_info *core_map = NULL;
static int num_core_maps = 0;

struct core_map_info *core_add_map_info(struct core_map_info **core_map_list,
					Elf *elf, GElf_Phdr *phdr)
{
	struct core_map_info *map;

	if (phdr->p_type != PT_LOAD || phdr->p_filesz == 0)
		return NULL;

	map = calloc(1, sizeof(*map));
	if (!map)
		return map;

	map->elf = elf;
	map->p_offset = phdr->p_offset;
	map->p_filesz = phdr->p_filesz;
	map->p_vaddr = phdr->p_vaddr;

	map->next = *core_map_list;
	*core_map_list = map;
	num_core_maps++;

	DEBUG("Added map for %#" PRIx64 " at %#" PRIx64 " with length %#" PRIu64 "\n",
	      map->p_vaddr, map->p_offset, map->p_filesz);
	return map;
}

struct core_map_info *find_map(struct core_map_info *map,
				      const GElf_Addr addr, size_t size)
{
	for (/* void */; map != NULL; map = map->next) {
//		DEBUG("p_vaddr=%#" PRIx64 ", addr=%#" PRIx64 ", size=%d\n",
//		      map->p_vaddr, addr, size);
		if (map->p_vaddr <= addr &&
		    map->p_vaddr + map->p_filesz > addr + size) {
			DEBUG("Found the correct map (%#" PRIx64 ", %#" PRIx64 ", %#" PRIu64 ")\n",
			      map->p_vaddr, map->p_offset, map->p_filesz);
			return map;
		}
	}

	return NULL;
}

void get_map_data(struct core_map_info *core_map, const GElf_Addr addr,
			 size_t size, void *dest)
{
	struct core_map_info *map = find_map(core_map, addr, size);

	if (map) {
		off_t offset = addr - map->p_vaddr;
		Elf_Data *data = elf_getdata_rawchunk(map->elf,
						      map->p_offset,
						      map->p_filesz,
						      ELF_T_BYTE);
		memcpy(dest, data->d_buf + offset, size);
	} else
		memset(dest, 0, size);
}

